import json
import logging.config
import unittest
from pathlib import Path
from typing import Dict
from pyld import jsonld

from obasparql.static import GET_ALL_USER_QUERY, GET_ONE_USER_QUERY
from obasparql.utils import generate_graph
from test.settings import GRAPH_BASE, ENDPOINT
import unittest

from obasparql import QueryManager
from test.settings import QUERY_DIRECTORY, CONTEXT_DIRECTORY, QUERIES_TYPES
from obasparql.query_manager import dispatch_sparql_query

MINT_USERNAME = generate_graph(GRAPH_BASE, "mint@isi.edu")


class TestFrame(unittest.TestCase):
    logger = logging.getLogger('testing')

    @staticmethod
    def write_json_to_file(data, filepath):
        with open(filepath, 'w') as f:
            json.dump(data, f, indent=4)

    def setUp(self):
        self.query_manager = QueryManager(queries_dir=QUERY_DIRECTORY,
                                          context_dir=CONTEXT_DIRECTORY,
                                          queries_types=QUERIES_TYPES)

    def test_frame_author(self):
        """
        This tests query a resource with type *MODEL*
        The properties: author and hasContactPerson have the same range: Person or Organization
        IRI: https://w3id.org/okn/o/sd#author
        IRI: https://w3id.org/okn/o/sd#hasContactPerson
        Returns:

        """
        owl_class_name = "Model"
        owl_class_uri = "https://w3id.org/okn/o/sdm#Model"
        resource_uri = "https://w3id.org/okn/i/mint/CYCLES"
        query_type = GET_ONE_USER_QUERY

        #grlc args
        request_args: Dict[str, str] = {
            "resource": resource_uri,
            "g": MINT_USERNAME
        }
        response = self.query_manager.obtain_query(query_directory=owl_class_name,
                                              owl_class_uri=owl_class_uri,
                                              query_type=query_type,
                                              endpoint=ENDPOINT,
                                              request_args=request_args)
        self.assertTrue(response)
        for author in response[0]["author"]:
            self.assertIsInstance(author, dict)
        for author in response[0]["hasContactPerson"]:
            self.assertIsInstance(author, dict)


if __name__ == '__main__':
    unittest.main()



import json
import os
from pathlib import Path

import requests
from SPARQLWrapper.SPARQLExceptions import EndPointInternalError, QueryBadFormed, Unauthorized, EndPointNotFound
from pyld import jsonld
from rdflib import Graph
from grlc import gquery
from SPARQLWrapper import SPARQLWrapper, POST, RDFXML, JSONLD
import re
import logging.config

EMBED_OPTION = "@always"

glogger = logging.getLogger("grlc")
logger = logging.getLogger('oba')


def convert_snake(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


class QueryManager:
    def __init__(self, **kwargs):
        """
        Load the queries template from the directory
        :param kwargs: contains the queries and context directories
        :type kwargs: dict
        """
        logger.debug("setting contexts")
        self.kwargs = kwargs
        queries_dir = Path(kwargs["queries_dir"])
        context_dir = Path(kwargs["context_dir"])
        default_dir = queries_dir / "_default_"

        # Obtain default queries
        queries = self.read_template(default_dir)
        setattr(self, "_default_", queries)

        # Overwrite default queries by class
        for owl_class in os.listdir(queries_dir):
            queries = self.read_template(queries_dir / owl_class)
            setattr(self, owl_class, self._default_)
            for key, value in queries.items():
                k = getattr(self, owl_class)
                k[key] = queries[key]
        for query_name, query_sparql in queries.items():
            glogger.debug(query_name)
            glogger.debug(query_sparql)
        # Fix: oba needs key as camelcase and snake_case
        temp_context = json.loads(self.read_context(context_dir / "context.json"))["@context"]
        self.context = temp_context.copy()
        for key, value in temp_context.items():
            key_snake = convert_snake(key)
            self.context[key] = value
            if key_snake != key:
                self.context[key_snake] = value
        self.context = {"@context": self.context}

    @staticmethod
    def insert_query(endpoint, request_args):
        query_string = f'{request_args["prefixes"]}' \
                       f'INSERT DATA {{ GRAPH <{request_args["g"]}> ' \
                       f'{{ {request_args["triples"]} }} }}'
        sparql = SPARQLWrapper(endpoint)
        sparql.setMethod(POST)
        try:
            sparql.setQuery(query_string)
            glogger.debug("insert_query: {}".format(query_string))
            sparql.query()
        except:
            glogger.error("Exception occurred", exc_info=True)
            return False
        return True

    @staticmethod
    def delete_query(endpoint, request_args):
        sparql = SPARQLWrapper(endpoint)
        sparql.setMethod(POST)
        query_string = f'' \
                       f'DELETE WHERE {{ GRAPH <{request_args["g"]}> ' \
                       f'{{ <{request_args["resource"]}> ?p ?o . }} }}'

        try:
            glogger.info("deleting {}".format(request_args["resource"]))
            glogger.debug("deleting: {}".format(query_string))
            sparql.setQuery(query_string)
            sparql.query()
        except Exception as e:
            glogger.error("Exception occurred", exc_info=True)
            return "Error delete query", 405, {}

        if request_args["delete_incoming_relations"]:
            query_string_reverse = f'' \
                                   f'DELETE WHERE {{ GRAPH <{request_args["g"]}> ' \
                                   f'{{ ?s ?p <{request_args["resource"]}>  }} }}'
            try:
                glogger.info("deleting incoming relations {}".format(request_args["resource"]))
                glogger.debug("deleting: {}".format(query_string_reverse))
                sparql.setQuery(query_string_reverse)
                sparql.query()
            except Exception as e:
                glogger.error("Exception occurred", exc_info=True)
                return "Error delete query", 405, {}

        return "Deleted", 202, {}

    def obtain_query(self, query_directory, owl_class_uri, query_type, endpoint, request_args=None, formData=None,
                     auth={}):
        """
        Given the owl_class and query_type, load the query template.
        Execute the query on the remote endpoint.
        :param formData:
        :type formData:
        :param query_directory:
        :type query_directory:
        :param query_type: The type of query. Required to load the query template.
        :type query_type: string
        :param endpoint: The url of the SPARQL endpoint
        :type endpoint: string
        :param request_args:
        :type request_args:
        :return: Framed JSON
        :rtype: string
        """
        query_template = getattr(self, query_directory)[query_type]
        result = dispatch_sparql_query(raw_sparql_query=query_template, request_args=request_args,
                                       endpoint=endpoint, return_format=JSONLD, auth=auth)
        logger.debug("response: {}".format(result))
        if "resource" in request_args:
            return self.frame_results(result, owl_class_uri, request_args["resource"])
        return self.frame_results(result, owl_class_uri)

    def frame_results(self, resp, owl_class_uri, owl_resource_iri=None):
        """
        Generate the framed using the owl_class.
        Frame the response and returns it.
        :param resp: JSON response from SPARQL
        :type resp: string
        :param owl_class_uri: OWL class uri
        :type owl_class: string
        :return: Framed JSON
        :rtype: string

        Args:
            owl_resource_iri:
        """
        try:
            triples = json.loads(resp)
        except Exception:
            glogger.error("json serialize failed", exc_info=True)
            return []
        frame = self.context.copy()
        frame['@type'] = owl_class_uri
        results = {"@graph": triples, "context": self.context.copy()}

        if owl_resource_iri is not None:
            frame['@id'] = owl_resource_iri

        framed = jsonld.frame(results, frame, {"embed": ("%s" % EMBED_OPTION)})
        if '@graph' in framed:
            return framed['@graph']
        else:
            return []

    @staticmethod
    def read_context(context_file):
        """
        Read the context file
        :param context_file: Absolute path of the file
        :type context_file: string
        :return: Contents of the file
        :rtype: string
        """
        with open(context_file, 'r') as reader:
            return reader.read()

    @staticmethod
    def read_template(owl_class_dir):
        queries = {}
        for file_query in os.listdir(owl_class_dir):
            filename, file_extension = os.path.splitext(file_query)
            if file_extension == ".rq":
                with open(owl_class_dir / file_query, 'r') as reader:
                    key_name = filename
                    queries[key_name] = reader.read()
        return queries


def dispatch_sparql_query(raw_sparql_query, request_args, endpoint, return_format, auth={}):
    query_metadata = gquery.get_metadata(raw_sparql_query, endpoint)
    rewritten_query = query_metadata['query']
    # Rewrite query using parameter values
    if query_metadata['type'] == 'SelectQuery' or query_metadata['type'] == 'ConstructQuery':
        try:
            rewritten_query = gquery.rewrite_query(query_metadata['original_query'], query_metadata['parameters'],
                                                   request_args)
        except Exception as e:
            logger.error("Parameters expected: {} ".format(query_metadata['parameters']))
            logger.error("Parameters given: {} ".format(request_args))
            raise e
    # Rewrite query using pagination
    if query_metadata['type'] == 'SelectQuery' and 'pagination' in query_metadata:
        try:
            rewritten_query = gquery.paginate_query(rewritten_query, query_metadata['pagination'], request_args)
        except Exception as e:
            logger.error("Parameters expected: {} ".format(query_metadata['parameters']))
            logger.error("Parameters given: {} ".format(request_args))
            raise e

    sparql = SPARQLWrapper(endpoint)
    sparql.setQuery(rewritten_query)
    sparql.setReturnFormat(return_format)
    try:
        results = sparql.query().convert().serialize(format=return_format)
    except EndPointInternalError as e:
        logger.error(e, exc_info=True)
    except EndPointInternalError as e:
        logger.error(e, exc_info=True)
    except QueryBadFormed as e:
        logger.error(e, exc_info=True)
    except Unauthorized as e:
        logger.error(e, exc_info=True)
    except EndPointNotFound as e:
        logger.error(e, exc_info=True)
    return results



import json
import logging
import unittest
from typing import Dict

from SPARQLWrapper import JSONLD

from obasparql.query_manager import dispatch_sparql_query, QueryManager
from obasparql.static import GET_ONE_USER_QUERY, GET_ALL_USER_QUERY, GET_ONE_QUERY
from obasparql.utils import generate_graph
from test.settings import *

logger = logging.getLogger('testing')
MINT_USERNAME = generate_graph(GRAPH_BASE, "mint@isi.edu")


class TestQueryManager(unittest.TestCase):
    def setUp(self):
        self.query_manager_model_catalog = QueryManager(queries_dir=QUERY_DIRECTORY,
                                                        context_dir=CONTEXT_DIRECTORY,
                                                        queries_types=QUERIES_TYPES)

        self.query_manager_dbpedia = QueryManager(queries_dir=QUERY_DIRECTORY_DBPEDIA,
                                                  context_dir=CONTEXT_DIRECTORY_DBPEDIA,
                                                  queries_types=QUERIES_TYPES)

    def test_obtain_query_get_one(self):
        """
        Test to obtain one resource by the uri
        """
        owl_class_name = "Band"
        owl_class_uri = "http://dbpedia.org/ontology/Band"
        resource_uri = "http://dbpedia.org/resource/Indemnity_Act_1717"
        query_type = GET_ONE_QUERY

        # grlc args
        request_args: Dict[str, str] = {
            "resource": resource_uri,
        }

        resource = self.query_manager_dbpedia.obtain_query(query_directory=owl_class_name,
                                                           owl_class_uri=owl_class_uri,
                                                           query_type=query_type,
                                                           endpoint=ENDPOINT_DBPEDIA,
                                                           request_args=request_args)

        self.assertTrue(resource)

    def test_obtain_query_get_one_user(self):
        """
        Test to obtain one resource by the uri
        """
        owl_class_name = "Model"
        owl_class_uri = "https://w3id.org/okn/o/sdm#Model"
        resource_uri = "https://w3id.org/okn/i/mint/CYCLES"
        query_type = GET_ONE_USER_QUERY

        # grlc args
        request_args: Dict[str, str] = {
            "resource": resource_uri,
            "g": MINT_USERNAME
        }

        resource = self.query_manager_model_catalog.obtain_query(query_directory=owl_class_name,
                                                                 owl_class_uri=owl_class_uri,
                                                                 query_type=query_type,
                                                                 endpoint=ENDPOINT,
                                                                 request_args=request_args)

        self.assertTrue(resource)

    def test_obtain_query_get_one_user_region_case(self):
        """
        Test to obtain one resource by the uri
        """
        owl_class_name = "Region"
        owl_class_uri = "https://w3id.org/okn/o/sdm#Region"
        resource_uri = "https://w3id.org/okn/i/mint/Travis"
        query_type = GET_ONE_USER_QUERY

        # grlc args
        request_args: Dict[str, str] = {
            "resource": resource_uri,
            "g": MINT_USERNAME
        }

        resource = self.query_manager_model_catalog.obtain_query(query_directory=owl_class_name,
                                                                 owl_class_uri=owl_class_uri,
                                                                 query_type=query_type,
                                                                 endpoint=ENDPOINT,
                                                                 request_args=request_args)

        self.assertEqual(resource_uri, resource[0]["id"])

    def test_dispatch_sparqlquery(self):
        endpoint = "http://dbpedia.org/sparql"
        query_template = '''
    PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
    CONSTRUCT {
        <http://dbpedia.org/resource/Indemnity_Act_1717> ?predicate ?prop .
        ?prop a ?type .
        ?prop rdfs:label ?label
    }
    WHERE {
        <http://dbpedia.org/resource/Indemnity_Act_1717> ?predicate ?prop
        OPTIONAL {
            ?prop  a ?type
            OPTIONAL {
                ?prop rdfs:label ?label
            }
        }
    }
            '''
        results = dispatch_sparql_query(raw_sparql_query=query_template, request_args={}, endpoint=endpoint,
                                        return_format=JSONLD)
        self.assertIsNotNone(json.loads(results))

    def test_dispatch_sparqlquery_model_catalog(self):
        """
        Testing to get the resource Travis
        Travis is a Region
        Returns:
        """
        owl_class_name = "Region"
        owl_resource_iri = "https://w3id.org/okn/i/mint/United_States"
        query_directory = owl_class_name
        query_type = GET_ONE_USER_QUERY

        endpoint = QUERY_ENDPOINT
        request_args: Dict[str, str] = {
            "resource": owl_resource_iri,
            "g": MINT_USERNAME
        }

        query_template = getattr(self.query_manager_model_catalog, query_directory)[query_type]

        results = dispatch_sparql_query(raw_sparql_query=query_template,
                                     request_args=request_args,
                                     endpoint=endpoint,
                                     return_format=JSONLD)
        self.assertIsNotNone(json.loads(results))

    def test_framed_get_one(self):
        owl_class_uri = "https://w3id.org/okn/o/sdm#Region"
        owl_resource_uri = "https://w3id.org/okn/i/mint/Travis"
        response = '''{
  "@graph" : [ {
    "@id" : "https://w3id.org/okn/i/mint/Texas",
    "@type" : "https://w3id.org/okn/o/sdm#Region",
    "label" : "Texas (USA)"
  }, {
    "@id" : "https://w3id.org/okn/i/mint/Travis",
    "@type" : "https://w3id.org/okn/o/sdm#Region",
    "label" : "Travis",
    "description" : "Travis (Texas)",
    "partOf" : "https://w3id.org/okn/i/mint/Texas"
  } ],
  "@context" : {
    "label" : {
      "@id" : "http://www.w3.org/2000/01/rdf-schema#label"
    },
    "partOf" : {
      "@id" : "https://w3id.org/okn/o/sdm#partOf",
      "@type" : "@id"
    },
    "description" : {
      "@id" : "https://w3id.org/okn/o/sd#description"
    },
    "sd" : "https://w3id.org/okn/o/sd#",
    "rdfs" : "http://www.w3.org/2000/01/rdf-schema#"
  }
}'''

        framed = self.query_manager_model_catalog.frame_results(response, owl_class_uri, owl_resource_uri)
        self.assertEqual(owl_resource_uri, framed[0]["id"])

    def test_framed_get_one_reflexive(self):
        owl_class_uri = "https://w3id.org/okn/o/sdm#Region"
        owl_resource_uri = "https://w3id.org/okn/i/mint/United_States"
        response = '''{
  "@id" : "https://w3id.org/okn/i/mint/United_States",
  "@type" : "https://w3id.org/okn/o/sdm#Region",
  "label" : "United States of America",
  "description" : "The United States of America (U.S.A. or USA), commonly known as the United States (U.S. or US) or America, is a country comprising 50 states, a federal district, five major self-governing territories, and various possessions. At 3.8 million square miles (9.8 million km2), the United States is the world's third or fourth largest country by total area and is slightly smaller than the entire continent of Europe. With a population of over 327 million people, the U.S. is the third most populous country. The capital is Washington, D.C., and the most populous city is New York City. Most of the country is located contiguously in North America between Canada and Mexico.",
  "partOf" : "https://w3id.org/okn/i/mint/America",
  "@context" : {
    "partOf" : {
      "@id" : "https://w3id.org/okn/o/sdm#partOf",
      "@type" : "@id"
    },
    "description" : {
      "@id" : "https://w3id.org/okn/o/sd#description"
    },
    "label" : {
      "@id" : "http://www.w3.org/2000/01/rdf-schema#label"
    },
    "sd" : "https://w3id.org/okn/o/sd#",
    "rdfs" : "http://www.w3.org/2000/01/rdf-schema#"
  }
}'''

        framed = self.query_manager_model_catalog.frame_results(response, owl_class_uri, owl_resource_uri)
        self.assertEqual(owl_resource_uri, framed[0]["id"])


if __name__ == '__main__':
    unittest.main()



import unittest
from typing import Dict

from obasparql.static import GET_ALL_USER_QUERY, GET_ONE_USER_QUERY
from test.settings import GRAPH_BASE, ENDPOINT
import unittest

from obasparql import QueryManager
from test.settings import QUERY_DIRECTORY, CONTEXT_DIRECTORY, QUERIES_TYPES


class TestSum(unittest.TestCase):
    @staticmethod
    def generate_graph(username):
        return "{}{}".format(GRAPH_BASE, username)

    def setUp(self):
        self.query_manager = QueryManager(queries_dir=QUERY_DIRECTORY,
                                          context_dir=CONTEXT_DIRECTORY,
                                          queries_types=QUERIES_TYPES)
        username = "mint@isi.edu"
        self.username = self.generate_graph(username)

    def test_get_all(self):
        """
        Test to obtain all the resources related to type
        """
        owl_class_name = "ModelConfiguration"
        owl_class_uri = "https://w3id.org/okn/o/sdm#ModelConfiguration"
        query_type = GET_ALL_USER_QUERY

        grlc_request_args: Dict[str, str] = {
            "type": owl_class_uri,
            "g": self.username
        }


        results = self.query_manager.obtain_query(query_directory=owl_class_name,
                                        owl_class_uri=owl_class_uri,
                                        query_type=query_type,
                                        endpoint=ENDPOINT,
                                        request_args=grlc_request_args)

        self.assertIsNotNone(results)

    def test_get_one(self):
        """
        Test to obtain one resource by the uri
        """
        owl_class_name = "ModelConfiguration"
        owl_class_uri = "https://w3id.org/okn/o/sdm#ModelConfiguration"
        resource_uri = "https://w3id.org/okn/i/mint/pihm-v2"
        query_type = GET_ONE_USER_QUERY

        request_args: Dict[str, str] = {
            "resource": resource_uri,
            "g": self.username
        }

        resource = self.query_manager.obtain_query(query_directory=owl_class_name,
                                                   owl_class_uri=owl_class_uri,
                                                   query_type=query_type,
                                                   endpoint=ENDPOINT,
                                                   request_args=request_args)
        self.assertEqual(len(resource), 1)
        self.assertEqual(resource[0]["id"], resource_uri)

    def test_get_one_setup_custom(self):
        """
        Test to obtain one resource by the uri and a custon query
        """
        owl_class_name = "custom"
        resource_uri = "https://w3id.org/okn/i/mint/cycles-0.9.4-alpha-simple-pongo"
        resource_type_uri = "https://w3id.org/okn/o/sdm#ModelConfigurationSetup"
        query_type = "custom_modelconfigurationsetups"

        #grlc args
        request_args: Dict[str, str] = {
            "resource": resource_uri,
            "g": self.username
        }

        resource = self.query_manager.obtain_query(query_directory=owl_class_name,
                                                   owl_class_uri=resource_type_uri,
                                                   query_type=query_type,
                                                   endpoint=ENDPOINT,
                                                   request_args=request_args)
        self.assertEqual(len(resource), 1)
        self.assertEqual(resource[0]["id"], resource_uri)

if __name__ == '__main__':
    unittest.main()
